--File name  : ChessClk.ex
--Info       : Simple Chess Clock / Countdown Timer
--Author     : Ernest Cheam
--           : Version 1.0
--Creation   : 8 September 1998  
--Contact    : nyamuk_58@hotmail.com      

include graphics.e 
include tsi_fnt1.e  -- thanks to Michael J. Sabal (added show_text)
include NK_funcs.e  -- thanks to Neil Kube

if graphics_mode(18) then      --18, 258 or 260
    abort(1)
end if

procedure beep() 
    sound(1000) pause(0.2) sound(0)
end procedure  

function p2xy(atom r, atom a)   -- polar to xy
   return {-r*sin(a),-r*cos(a)}
end function

constant PI = 3.14159265,
  colors={BRIGHT_BLUE,BRIGHT_GREEN,WHITE,YELLOW,GRAY},
		-- colors: player1,player2,text,expired,background
  cen={{120,360},{520,360}},  	  -- centers of clock
  r=100,			  -- radius of clock
  x={1,51},			  -- x for text
  digits={{0,220},{400,220}}	  -- location for digital display

atom t
integer rtdef,soundon,p,paused,c,hundredth
sequence tr,tp,tt

procedure change()
   if paused then 
     tp[1]=tr[1] tp[2]=tr[2] show_text(8,2,colors[3]," PAUSED ")
   else 
     tt[p]=t tp[3-p]=tr[3-p] show_text(8,2,colors[p],"PLAYER " & '0'+p)
   end if
   position(13,x[p]) puts(1,'') position(13,x[3-p]) puts(1,' ')
end procedure

procedure showtime(atom rt,integer k)
integer col,h
  col=colors[k]
  if rt<=3600 then -- don't show hand for more than 1 hour remaining
    if rt>0 then 
      draw_line(colors[5],{cen[k],cen[k]+p2xy(r,rt/1800*PI)})
    else  -- expired
      draw_line(colors[4],{cen[k],cen[k]+p2xy(r,rt/1800*PI)})
      rt=-rt col=colors[4] if soundon then beep() end if
    end if
  end if
  h=floor(rt/3600)
  if h=0 then ShowDigital(8, {0, 0}, digits[k], 1.5, 1)	  -- don't show 0 hr
         else ShowDigital(h, {col, 0}, digits[k], 1.5, 1) -- show hour
  end if
  ShowDigital(remainder(floor(rt/60),60),{col,0},digits[k]+{40,0},1,2) -- min
  ShowDigital(remainder(floor(rt),60),{col,0},digits[k]+{90,0},1,2) -- second
  if hundredth then  -- hundredth second
    ShowDigital(remainder(floor(rt*100),100),{col,0},digits[k]+{140,0},0.8,2)
  end if
end procedure

procedure togglehundredth()
  hundredth=not hundredth
  if hundredth then showtime(tr[1],1) showtime(tr[2],2)
  else  ShowDigital(88,{0,0},digits[1]+{140,0},0.8,2) 
        ShowDigital(88,{0,0},digits[2]+{140,0},0.8,2)
  end if
end procedure

procedure rtinc(integer a, integer k) -- manually adjust remaining time
  if a>0 and tr[k]<=3600 then
    if tr[k]>0 then draw_line(colors[k],{cen[k],cen[k]+p2xy(r,tr[k]/1800*PI)})
    else draw_line(colors[5],{cen[k],cen[k]+p2xy(r,tr[k]/1800*PI)})
    end if
  end if
  tr[k]=tr[k]+a 	-- no check for limits! But shouldn't be a problem
  tt[k]=t tp[k]=tr[k] showtime(tr[k],k) 
end procedure

procedure showstatus()
  text_color(colors[3]) 
  position(14,30) printf(1,"RT def=%d mins.",{rtdef}) 
  position(15,30) puts(1,"Sound ")
  if soundon then puts(1,"ON ") else puts(1,"OFF") end if
  position(16,30) puts(1,"Press h for help")
end procedure

procedure reset()
sequence mc
  tr={rtdef*60,rtdef*60}	-- reset to initial remaining times
  p=1 paused=1
  t=time() tt={t,t} tp={0,0}
  clear_screen()
  showstatus()
  change() 
  for k = 1 to 2 do		-- draw clocks and static text
    ellipse(colors[5],1,cen[k]-r,cen[k]+r)
    for m = 0 to 59 do
      mc=cen[k]+p2xy(r+4,m/30*PI)  -- centerpoints of minutes indicators
      if remainder(m,5)=0 then ellipse(MAGENTA,1,mc-{2,2},mc+{2,2})
                          else ellipse(GRAY,1,mc-{1,1},mc+{1,1})
      end if
    end for
    for rt = 0 to min({tr[k],3600}) do  -- shade remaining time
      draw_line(colors[k],{cen[k],cen[k]+p2xy(r,rt/1800*PI)})  	
    end for
    text_color(colors[k])  
    position(13,x[k]+1) puts(1,"Player "& '0'+k & " Time remaining:")
    ellipse(colors[k],1,digits[k]+{80,7}-{1,1},digits[k]+{80,7}+{1,1})
    ellipse(colors[k],1,digits[k]+{80,17}-{1,1},digits[k]+{80,17}+{1,1})
    showtime(tr[k],k)
  end for
end procedure

procedure showhelp()
sequence helptext,z
  helptext={"    Press  to                          ",
            "    -----  ------------------------    ",	
            "    space  Continue                    ",
            "    1/2    Switch to player 1/2        ",  
            "    p      pause                       ",
            "    r      reset                       ",
            "    s      toggle sound on/off         ",
            "    g/v    inc/dec default time  1 m   ",
            "    a/z    inc/dec player 1's by 1 s   ",
            "    k/m    inc/dec player 2's by 1 s   ",
            "    .      toggle hundredth second     ",
            "    ESC    quit                        "}
  z={152,0,463,190}
  text_color(CYAN)
  for i = 1 to 12 do position(i,20) puts(1,helptext[i])  end for
  draw_line(CYAN,{{z[1],z[2]},{z[3],z[2]},{z[3],z[4]},{z[1],z[4]},{z[1],z[2]}})
  while get_key()=-1 do  end while
  for i = 1 to 12 do position(i,20) puts(1,repeat(' ',37)) end for 
  draw_line(0,{{z[1],z[2]},{z[3],z[2]},{z[3],z[4]},{z[1],z[4]},{z[1],z[2]}})
  change()	
end procedure

-- main --
rtdef=60	-- PUT YOUR FAVORITE INITIAL REMAINING TIME HERE (IN MINUTES) --
soundon=1  
hundredth=0
reset()
while 1 do	-- main loop
  t=time()
  if not paused then tr[p]=tp[p]-t+tt[p] showtime(tr[p],p) end if
  c=get_key()
  if c!=-1 then
    if    c='1' then p=1 change()
    elsif c='2' then p=2 change()
    elsif c='p' then paused=not paused change()
    elsif c=' ' then 
      if paused then paused=not paused else p=3-p end if change()
    elsif c='a' then rtinc(1,1)
    elsif c='z' then rtinc(-1,1)
    elsif c='k' then rtinc(1,2)
    elsif c='m' then rtinc(-1,2)
    elsif c='s' then soundon=not soundon showstatus()
    elsif c='g' then if rtdef<599 then rtdef=rtdef+1 showstatus() end if
    elsif c='v' then if rtdef>1 then rtdef=rtdef-1 showstatus() end if
    elsif c='r' then reset()
    elsif c='.' then togglehundredth()
    elsif c='h' then showhelp()
    elsif c=27 then c=graphics_mode(-1) exit
    end if
  end if
end while